"""
Host
====================================
The Host class can be used to access information about an host through the
REST API (https://www.ntop.org/guides/ntopng/api/rest/api_v2.html).
"""

class Host:
    """
    Host provides information about hosts
    
    :param ntopng_obj: The ntopng handle
    """
    def __init__(self, ntopng_obj, ifid, ip, vlan=None):
        """
        Construct a new Host object

        :param ntopng_obj: The ntopng handle (Ntopng instance)
        :param ifid: The interface ID
        :type ifid: int
        :param ip: The host IP address
        :type ip: string
        :param vlan: The host VLAN ID (if any)
        :type vlan: int
        """
        self.ntopng_obj      = ntopng_obj
        self.ifid            = ifid
        self.ip              = ip
        self.vlan            = vlan
        self.rest_v2_url     = "/lua/rest/v2"
        self.rest_pro_v2_url = "/lua/pro/rest/v2"

    def get_host_data(self):
        """
        Return all available information about a single host
        
        :return: Information about the host
        :rtype: object
        """

        params = { "ifid": self.ifid, "host": self.ip }
        if(self.vlan is not None):
            params['vlan'] = self.vlan

        return(self.ntopng_obj.request(self.rest_v2_url + "/get/host/data.lua", params))

    def get_l7_stats(self):
        """
        Return statistics about Layer 7 protocols for the host
        
        :return: Layer 7 protocol statistics
        :rtype: object
        """
        params = { "ifid": self.ifid, "host": self.ip, "breed": True, "ndpi_category": True, "collapse_stats": False }
        if(self.vlan is not None):
            params['vlan'] = self.vlan

        return(self.ntopng_obj.request(self.rest_v2_url + "/get/host/l7/stats.lua", params))

    def get_dscp_stats(self, direction_rcvd):
        """
        Return statistics about DSCP per traffic direction for an host
        
        :param direction_rcvd: The traffic direction (True for received traffic, False for sent)
        :type direction_rcvd: boolean
        :return: DSCP statistics
        :rtype: object
        """
        if(direction_rcvd):
            direction = "recvd"
        else:
            direction = "sent"

        params = { "ifid": self.ifid, "host": self.ip, "direction": direction }
        if(self.vlan is not None):
            params['vlan'] = self.vlan

        return(self.ntopng_obj.request(self.rest_v2_url + "/get/host/dscp/stats.lua", params))

    def get_active_flows_paginated(self, currentPage, perPage):
        """
        Retrieve the (paginated) list of active flows for the specified interface and host
        
        :param currentPage: The current page
        :type currentPage: int
        :param perPage: The number of results per page
        :type perPage: int
        :return: All active flows
        :rtype: array
        """
        return(self.ntopng_obj.request(self.rest_v2_url + "/get/flow/active.lua", {"ifid": self.ifid, "host": self.ip, "vlan": self.vlan, "currentPage": currentPage, "perPage": perPage}))

    def self_test(self):
        try:
            print("Host Data ----------------------------")
            print(self.get_host_data())
            print("L7 Stats ----------------------------")
            print(self.get_l7_stats())
            print("Active Flows ----------------------------")
            print(self.get_active_flows_paginated(1, 10))
            print("DSCP Stats (RX) ----------------------------")
            print(self.get_dscp_stats(True))
            print("DSCP Stats (TX) ----------------------------")
            print(self.get_dscp_stats(False))
            print("----------------------------")
        except:
            raise ValueError("Invalid interface ID or host specified")



"""
Ntopng
====================================
The Ntopng class stores information for accessing the ntopng instance (IP and credentials)
and provides global traffic information and constants (interfaces, alert types, etc).
"""

import requests
import json
from requests.auth import HTTPBasicAuth

from ntopng.interface import Interface
from ntopng.historical import Historical

class Ntopng:        
    """
    Ntopng provides information about global data (e.g. list of interfaces) and consts (e.g. alert types)
    
    :param ntopng_obj: The ntopng handle
    """
    def __init__(self, username = None, password = None, auth_token = None, url = "http://localhost:3000"):
        """
        Construct a new Ntopng object
        
        :param username: The ntopng username (leave empty if token authentication is used)
        :type username: string
        :param password: The ntopng password (leave empty if token authentication is used)
        :type password: string
        :param auth_token: The authentication token (leave empty if username/password authentication is used)
        :type auth_token: int
        :param url: The ntopng URL (e.g. http://localhost:3000)
        :type url: string
        """
        self.url             = url
        self.rest_v2_url     = "/lua/rest/v2"
        self.rest_pro_v2_url = "/lua/pro/rest/v2"

        if(auth_token != None):
            self.auth_token = auth_token
            self.username = None
            self.password = None
        else:
            self.username   = username
            self.password   = password
            self.auth_token = None
            
        self.debug = False
        
        # self_test
        try:
            url = self.url + "/lua/rest/v2/connect/test.lua"

            response = self.issue_request(url, None)
                
            if(not(response.headers['Content-Type'].startswith('application/json'))):
                raise ValueError("Invalid credentials or URL specified")
        except:
            raise ValueError("Invalid credentials or URL specified")

    def get_url(self):
        return self.url

    def issue_request(self, url, params):
        if(self.debug):
            print("Requesting [GET]: "+url)
            print(params)

        if(self.auth_token != None):
            response = requests.get(url, auth = None, headers = { "Authorization" : "Token " + self.auth_token }, params = params)
        else:
            response = requests.get(url, auth = HTTPBasicAuth(self.username, self.password), params = params)

        if(self.debug):
            print("Elapsed time: " + str(response.elapsed))
            
        return(response)

    def issue_post_request(self, url, params):
        if(self.debug):
            print("Requesting [POST]: " + url)
            print(params)
        
        if(self.auth_token != None):
            response = requests.post(url, auth = None, headers = { "Authorization" : "Token " + self.auth_token, "Content-Type" : "application/json" }, json = params)
        else:
            response = requests.post(url, auth = HTTPBasicAuth(self.username, self.password), headers = { "Content-Type" : "application/json" }, json = params)

        if(self.debug):
            print("Elapsed time: " + str(response.elapsed))
            
        return(response)

    def enable_debug(self):
        self.debug = True
 
    # internal method used to issue requests
    def request(self, url, params):
        api_url = self.url + url

        if(self.debug):
            print(params)
            
        response = self.issue_request(api_url, params)
            
        if response.status_code != 200:
            print(api_url)
            print(params)
            print("Bad response code " + str(response.status_code))
            raise Exception("Bad response code " + str(response.status_code))

        response = response.json()

        return response['rsp']

    # internal method used to issue requests
    def post_request(self, url, params):
        api_url = self.url + url

        if(self.debug):
            print(params)
            
        response = self.issue_post_request(api_url, params)
            
        if response.status_code != 200:
            print(api_url)
            print(params)
            print("Bad response code " + str(response.status_code))
            raise Exception("Bad response code " + str(response.status_code))

        response = response.json()

        return response['rsp']

    def get_alert_types(self):
        """
        Return all alert types
        
        :return: The list of alert types
        :rtype: array
        """
        return(self.request(self.rest_v2_url + "/get/alert/type/consts.lua", None))

    def get_alert_severities(self):
        """
        Return all severities
        
        :return: The list of severities
        :rtype: array
        """
        return(self.request(self.rest_v2_url + "/get/alert/severity/consts.lua", None))

    def get_interface(self, ifid):
        """
        Return an Interface instance
        
        :param ifid: The interface ID
        :type ifid: int
        :return: The interface instance
        :rtype: ntopng.Interface
        """ 
        return Interface(self, ifid)

    def get_historical_interface(self, ifid):
        """
        Return an Historical handle for an interface
        
        :param ifid: The interface ID
        :type ifid: int
        :return: The historical handle
        :rtype: ntopng.Historical
        """ 
        return Historical(self, ifid)

    def get_interfaces_list(self):
        """
        Return all available interfaces
        
        :return: The list of interfaces
        :rtype: array
        """
        return(self.request(self.rest_v2_url + "/get/ntopng/interfaces.lua", None))

    def get_host_interfaces_list(self, host):
        """
        Return all ntopng interfaces for a given host
        
        :param host: The host
        :type host: string
        :return: List of interfaces
        :rtype: array
        """
        return(self.request(self.rest_v2_url + "/get/host/interfaces.lua", { "host": host }))

    def self_test(self):
        try:
            print("Alert Types ----------------------------")
            print(self.get_alert_types())
            print("Severities ----------------------------")
            print(self.get_alert_severities())
            print("Interfaces List ----------------------------")
            print(self.get_interfaces_list())
            print("----------------------------")
        except:
            raise ValueError("Unable to retrieve information")   



"""
Report
====================================
The Report class can be used to generate and send a report (e.g. PDF) with information about traffic on an interface or a host
"""

import os
import sys
import time
import uuid
import getopt
import base64
import pandas as pd
from fpdf import FPDF
import plotly.figure_factory as ff
import plotly.graph_objects as go

from ntopng.logo import *
from ntopng.host import Host
from ntopng.historical import Historical

class Report:
    """
    Report provides utility functions to generate reports
    
    :param ntopng_obj: The ntopng handle
    """

    def __init__(self, ntopng_obj, ifid, host=None):
        """
        Construct a new Report object
        
        :param ntopng_obj: The ntopng handle
        :type ntopng_obj: Ntopng
        :param ifid: The interface ID
        :type ifid: int
        :param host: The host IP address (optional)
        :type host: string
        """ 

        self.created_files = []

        self.ntopng_obj = ntopng_obj
        self.ifid = ifid
        self.host = host

        try:
            self.interface = self.ntopng_obj.get_interface(ifid)
            self.historical = self.ntopng_obj.get_historical_interface(ifid)
    
        except ValueError as e:
            print(e)
            os._exit(-1)
  
    def gen_tmp_file_name(self, ext):
        filename = str(uuid.uuid4())
        path = "/tmp/" + filename + "." + ext
        self.created_files.append(path)
        return path

    def clear_cache(self):
        for file in self.created_files:
            self.delete_file(file)
        self.created_files = []
 
    def get_alerts_by_severity(self, epoch_begin, epoch_end):
        """
        Return a dataframe with alerts by severity
        """
        alerts_stats = self.historical.get_alerts_stats(epoch_begin, epoch_end, self.host)
        for stats in alerts_stats:
            # by severity
            if stats['name'] == 'top_alerts_by_severity':
                alerts_stats_df = pd.DataFrame(stats["value"])
                alerts_stats_df.drop(["key", "title"], axis= 1, inplace=True)
                alerts_stats_df = alerts_stats_df[["label", "severity_label"]]
                alerts_stats_df.rename(columns={"label": "Alert type", "severity_label": "Severity"}, inplace=True)

        if alerts_stats_df is None:
            alerts_stats_df = pd.DataFrame([])

        return alerts_stats_df

    def get_alerts_by_count(self, epoch_begin, epoch_end):
        """
        Return a dataframe with alerts by count
        """
        alerts_stats = self.historical.get_alerts_stats(epoch_begin, epoch_end, self.host)
        for stats in alerts_stats:
            # by count
            if stats['name'] == 'top_alerts_by_count':
                alerts_stats_df = pd.DataFrame(stats["value"])
                alerts_stats_df.drop(["key", "title"], axis= 1, inplace=True)
                alerts_stats_df['count'] = alerts_stats_df['count'].round(1)
                alerts_stats_df['count'] = alerts_stats_df['count'].astype(str) + " %"
                alerts_stats_df = alerts_stats_df[["label", "count"]]
                alerts_stats_df.rename(columns={"label": "Alert type", "count": "Occurrence %"}, inplace=True)

        if alerts_stats_df is None:
            alerts_stats_df = pd.DataFrame([])

        return alerts_stats_df
    
    def get_top_conversations(self, epoch_begin, epoch_end):
        """
        Return a dataframe with top clients and servers seen on the interface specified in the above script init data
        """
        
        try:
            top_conversations = self.historical.get_top_conversations(epoch_begin, epoch_end, self.host) 

            conversations = { 'srv_name': [], 'cli_name': [], 'srv_ip': [], 'cli_ip': [], 'bytes': [] }
            for conv in top_conversations:
                conversations['srv_name'].append(conv['srv_name'])
                conversations['cli_name'].append(conv['cli_name'])
                conversations['srv_ip'].append(conv['srv_ip']['ip'])
                conversations['cli_ip'].append(conv['cli_ip']['ip'])
                conversations['bytes'].append(conv['bytes'])
                
            top_conversations_df = pd.DataFrame(conversations)
            # reorder top_conversations_df columns
            top_conversations_df = top_conversations_df[["cli_name", "cli_ip", "srv_name", "srv_ip", "bytes"]]
            # rename top_conversations_df columns
            top_conversations_df.rename(columns={"cli_name": "Client Name", "cli_ip": "Client IP", "srv_name": "Server Name", "srv_ip": "Server IP", "bytes": "Traffic"}, inplace=True)
        except:
            print("Top Client/Server report not available in community mode")
            top_conversations_df = pd.DataFrame([])

        return top_conversations_df
   
    def get_upload_download(self, epoch_begin, epoch_end):
        """
        Return a list where => data[0] = uploaded MB, data[1] = downloaded MB for the interface specified in the above script init data
        """

        if self.host is not None:
            up_down = self.historical.get_host_timeseries_stats(self.host, "host:traffic", epoch_begin, epoch_end)
        else:
            up_down = self.historical.get_interface_timeseries_stats("iface:traffic_rxtx", epoch_begin, epoch_end)
        data = [up_down["by_serie"][0]["total"]/(1024*1024), up_down["by_serie"][1]["total"]/(1024*1024)] 
        return data
    
    def get_interface_data(self):
        """
        Return a dict with info about the interface specified in the above script init data
        """
    
        host_data = self.interface.get_data()
    
        host_info = {"flow_count": host_data["num_flows"], "local_hosts": host_data["num_local_hosts"], "hosts_count": host_data["num_hosts"], "local_hosts_anomalies": host_data["num_local_hosts_anomalies"], "alerted_flows": host_data["alerted_flows"], "num_flows": host_data["num_flows"],
                    "flows_warning": host_data["alerted_flows_warning"], "flows_error": host_data["alerted_flows_error"]}
        
        return host_info
    
    def get_interfaces_count(self):
        """
        Return number of interfaces
        """
        interfaces = self.ntopng_obj.get_interfaces_list()
        return len(interfaces)
    
    def delete_file(self, fname):
        """
        Deletes the table png from the current path
        :param fname: name of png to delete
        """
        # Relative path
        #current_dir = os.getcwd()
        #path = current_dir + "/" + fname

        # Absolute path
        path = fname
    
        try:
            os.path.exists(path)
            os.remove(path)
    
        except Exception as e:
            print(f"{path} does not exist")
            os._exit(-1)
    
    def df_to_table_png(self, df, fname, width, height):
        """
        Creates a png of the df specified with name
        :param df: dataframe to save to png
        :param fname: name of the file to save
        """
        fig = ff.create_table(df, colorscale=[[0, "#f3a114"], [.5, "#d9d9d9"], [1, "#ffffff"]])
        fig.update_layout(autosize=False, width=width, height=height)
        fig.write_image(str(fname), scale=2)
    
    def plot_traffic_chart(self, output_series_fname, epoch_begin, epoch_end):
        """
        Plots a png of the timeseries
        """
        if self.host is not None:
            up_down = self.historical.get_host_timeseries(self.host, "host:traffic", epoch_begin, epoch_end)
        else:
            up_down = self.historical.get_interface_timeseries("iface:traffic_rxtx", epoch_begin, epoch_end)
        up_down.reset_index(inplace=True)
        up_down['bytes_rcvd'] = up_down['bytes_rcvd'].mul(8)
        up_down['bytes_sent'] = up_down['bytes_sent'].mul(8)
        up_down['ts'] = up_down['index'].apply(lambda x: x.left)
        up_down['ts'] = pd.to_datetime(up_down['ts'], unit='s')
        
        layout = dict(plot_bgcolor='rgba(255,255,255, 0.3)')
        fig = go.Figure(layout=layout)
        fig.add_trace(go.Scatter(x=up_down["ts"], y=up_down["bytes_rcvd"],
                            mode='lines',
                            name='Received (bps)',
                            line_color='#f3a114'))
        fig.add_trace(go.Scatter(x=up_down["ts"], y=up_down["bytes_sent"],
                            mode='lines',
                            name='Sent (bps)',
                            line_color='#000000'))
    
        fig.write_image(output_series_fname, width=1280, height=720)
   
    def build(self, output_file): 
     
        # Last 24h interval
        actual_ts = int(time.time())
        yesterday = (actual_ts - 86400)
        epoch_begin = yesterday
        epoch_end = actual_ts
  
        # Alerts table (severity)
        alerts_by_severity_df = self.get_alerts_by_severity(epoch_begin, epoch_end)
        alerts_by_severity_df_fname = self.gen_tmp_file_name("png")
        self.df_to_table_png(alerts_by_severity_df, alerts_by_severity_df_fname, 450, 325)

        # Alerts table (count)
        alerts_by_count_df = self.get_alerts_by_count(epoch_begin, epoch_end)
        alerts_by_count_df_fname = self.gen_tmp_file_name("png")
        self.df_to_table_png(alerts_by_count_df, alerts_by_count_df_fname, 450, 325)
        
        # Top hosts table
        hosts_df = self.get_top_conversations(epoch_begin, epoch_end, )
        hosts_df_fname = self.gen_tmp_file_name("png")
        self.df_to_table_png(hosts_df, hosts_df_fname, 950, 325)
        
        # Interface info
        data_sent_rcvd = self.get_upload_download(epoch_begin, epoch_end)
        sent = data_sent_rcvd[0]
        received = data_sent_rcvd[1]
        interfaces_count = self.get_interfaces_count()
        interface_data = self.get_interface_data()
        flows = interface_data["num_flows"]
        host_num = interface_data["hosts_count"]
        local_hosts_num = interface_data["local_hosts"]
        flows_errors = interface_data["flows_error"]
        flows_warnings = interface_data["flows_warning"]
        
        # PDF creation
        
        margin = 10
        pw = 25 - 2*margin
        ch = 50

        pdf = FPDF()
        pdf.add_page()
        
        # Logo
        ntop_logo_fname = self.gen_tmp_file_name("png")
        with open(ntop_logo_fname, "wb") as fh:
            fh.write(base64.b64decode(ntop_logo_b64))
        pdf.image(ntop_logo_fname, x=140, y=15, w=68.2, h=17.8)
        
        # ntopng host info
        pdf.ln(9)
        pdf.set_font("Helvetica", "B", 18)
        pdf.set_text_color(r=0, g=0, b=0)
        if self.host is not None:
            pdf.cell(w=0, h=10, txt=f"Host {self.host} Report (24h)", ln=1)
        else:
            pdf.cell(w=0, h=10, txt="Interface Report (24h)", ln=1)

        pdf.set_font("Helvetica", "", 11)
        ntopng_url = self.ntopng_obj.get_url()
        pdf.cell(w=0, h=5, txt=f"ntopng URL: {ntopng_url}")
        pdf.ln(5)

        pdf.cell(w=0, h=5, txt=f"Date: {time.strftime('%d-%m-%Y %H:%M:%S', time.localtime(epoch_end))}")
        pdf.ln(5)

        pdf.cell(w=0, h=5, txt=f"Interface: {self.ifid} (of {interfaces_count} interfaces)")
        pdf.ln(5)
        
        # Alerts
        pdf.set_font("Helvetica", "B", 14)
        pdf.cell(w=0, h=10, txt="Top Alerts")

        pdf.image(alerts_by_severity_df_fname, x=10, y=55, w=90, h=65)
        pdf.image(alerts_by_count_df_fname, x=110, y=55, w=90, h=65)

        # Hosts
        pdf.ln(75)
        pdf.set_font("Helvetica", "B", 14)
        pdf.cell(w=0, h=10, txt="Top Contacts")

        pdf.image(hosts_df_fname, x=10, y=130, w=190, h=65)
        
        #  Traffic stats
        pdf.ln(77)
        pdf.set_font("Helvetica", "B", 14)
        pdf.cell(w=0, h=10, txt="Traffic Summary")
        pdf.ln(5)
        pdf.set_font("Helvetica", "", 11)
        """
        pdf.cell(w=(pw/2), h=(ch/4), txt=f"{flows} Active Flows")
        pdf.ln(5)
        pdf.cell(w=(pw/2), h=(ch/4), txt=f"{host_num} Hosts")
        pdf.ln(5)
        pdf.cell(w=(pw/2), h=(ch/4), txt=f"{local_hosts_num} Local Hosts")
        pdf.ln(5)
        """
        pdf.cell(w=(pw/2), h=(ch/4), txt=f"Total Sent/Received: {sent:.2f} / {received:.2f} MB")

        series_fname = self.gen_tmp_file_name("png")
        self.plot_traffic_chart(series_fname, epoch_begin, epoch_end)
        pdf.image(series_fname, x=10, y=210, w=190, h=85)
        
        pdf.output(f"{output_file}", "F")
        
        self.clear_cache()



from setuptools import setup

setup(
    name='ntopng',
    version='5.7.230213',
    description='ntopng Python package',
    url='https://github.com/ntop/ntopng',
    author='ntop',
    author_email='packager@ntop.org',
    license='GPL',
    packages=['ntopng'],
    install_requires=['requests', 'simplejson', 'numpy', 'pandas', 'kaleido', 'fpdf', 'plotly', 'redmail'],
 )



"""
Interface
====================================
The Interface class can be used to access information about interface statistics through the
REST API (https://www.ntop.org/guides/ntopng/api/rest/api_v2.html).
"""

from ntopng.host import Host
from ntopng.historical import Historical

class Interface:
    """
    Interface provides information about a Network interface
    
    :param ntopng_obj: The ntopng handle
    :param ifid: The interface ID
    """

    def __init__(self, ntopng_obj, ifid):
        """
        Construct a new Interface object
        
        :param ntopng_obj: The ntopng handle
        :type ntopng_obj: Ntopng
        :param ifid: The interface ID
        :type ifid: int
        """ 
        self.ntopng_obj = ntopng_obj
        self.ifid = ifid
        self.rest_v2_url = "/lua/rest/v2"
        self.rest_pro_v2_url = "/lua/pro/rest/v2"
        
    def get_data(self):
        """
        Return information about a Network interface
        
        :return: Information about the interface
        :rtype: object
        """
        return(self.ntopng_obj.request(self.rest_v2_url + "/get/interface/data.lua", {"ifid": self.ifid}))
    
    def get_broadcast_domains(self):
        """
        Return information about broadcast domains on an interface
        
        :return: Information about broadcast domains
        :rtype: object
        """
        return(self.ntopng_obj.request(self.rest_v2_url + "/get/interface/bcast_domains.lua", {"ifid": self.ifid}))
    
    def get_address(self):
        """
        Return the interface IP address(es)
        
        :return: The interface address(es)
        :rtype: array
        """
        return(self.ntopng_obj.request(self.rest_v2_url + "/get/interface/address.lua", {"ifid": self.ifid}))

    def get_l7_stats(self, max_num_results):
        """
        Return statistics about Layer 7 protocols seen on an interface
        
        :param max_num_results: The maximum number of results to limit the output
        :type max_num_results: int
        :return: Layer 7 protocol statistics
        :rtype: object
        """
        return(self.ntopng_obj.request(self.rest_v2_url + "/get/interface/l7/stats.lua",
                              {"ifid": self.ifid,
                               'ndpistats_mode': 'count',
                               'breed': True,
                               'ndpi_category': True,
                               'all_values' : True,
                               'max_values': max_num_results,
                               'collapse_stats': False
                               }))
    
    def get_dscp_stats(self):
        """
        Return statistics about DSCP
        
        :return: DSCP statistics
        :rtype: object
        """
        return(self.ntopng_obj.request(self.rest_v2_url + "/get/interface/dscp/stats.lua", {"ifid": self.ifid}))

    def get_host(self, ip, vlan=None):
        """
        Return an Host instance
        
        :param ifid: The interface ID
        :type ifid: int
        :param ip: The host IP address
        :type ip: string
        :param vlan: The host VLAN ID (if any)
        :type vlan: int
        :return: The host instance
        :rtype: ntopng.Host
        """ 
        return Host(self.ntopng_obj, self.ifid, ip, vlan)

    def get_active_hosts(self):
        """
        Retrieve the list of active hosts for the specified interface
        
        :return: All active hosts
        :rtype: array
        """
        rsp = self.ntopng_obj.request(self.rest_v2_url + "/get/host/active.lua", {"ifid": self.ifid, "all": "true"})
        return rsp["data"]

    def get_active_hosts_paginated(self, currentPage, perPage):
        return(self.ntopng_obj.request(self.rest_v2_url + "/get/host/active.lua", {"ifid": self.ifid, "currentPage": currentPage, "perPage": perPage}))

    def get_top_local_talkers(self):
        """
        Return Top Local hosts generating more traffic
        
        :return: The top local hosts
        :rtype: array
        """
        return(self.ntopng_obj.request(self.rest_pro_v2_url + "/get/interface/top/local/talkers.lua", { "ifid": self.ifid }))

    def get_top_remote_talkers(self):
        """
        Return Top Remote hosts generating more traffic
        
        :return: The top remote hosts
        :rtype: array
        """
        return(self.ntopng_obj.request(self.rest_pro_v2_url + "/get/interface/top/remote/talkers.lua", { "ifid": self.ifid }))

    def get_active_flows_paginated(self, currentPage, perPage):
        """
        Retrieve the (paginated) list of active flows for the specified interface
        
        :param currentPage: The current page
        :type currentPage: int
        :param perPage: The number of results per page
        :type perPage: int
        :return: All active flows
        :rtype: array
        """
        return(self.ntopng_obj.request(self.rest_v2_url + "/get/flow/active.lua", {"ifid": self.ifid, "currentPage": currentPage, "perPage": perPage}))

    def get_active_l4_proto_flow_counters(self):
        """
        Return statistics about active flows per Layer 4 protocol on an interface
        
        :return: Layer 4 protocol flows statistics
        :rtype: object
        """
        return(self.ntopng_obj.request(self.rest_v2_url + "/get/flow/l4/counters.lua", {"ifid": self.ifid }))

    def get_active_l7_proto_flow_counters(self):
        """
        Return statistics about active flows per Layer 7 protocol on an interface
        
        :return: Layer 7 protocol flows statistics
        :rtype: object
        """
        return(self.ntopng_obj.request(self.rest_v2_url + "/get/flow/l7/counters.lua", {"ifid": self.ifid }))

    def get_historical(self):
        """
        Return an Historical handle for the interface
        
        :return: The historical handle
        :rtype: ntopng.Historical
        """ 
        return Historical(self.ntopng_obj, self.ifid)

    def self_test(self):
        print(self.get_data())
        try:
            print("Broadcast Domains ----------------------------")
            print(self.get_broadcast_domains())
            print("Address ----------------------------")
            print(self.get_address())
            print("L7 Stats ----------------------------")
            max_num_records = 100
            print(self.get_l7_stats(max_num_records))
            print("DSCP Stats ----------------------------")
            print(self.get_dscp_stats())
            print("Active Hosts ----------------------------")
            print(self.get_active_hosts())
            print("Active Hosts (100) ----------------------------")
            print(self.get_active_hosts_paginated(1, 100))
            print("Top Local Talkers ----------------------------")
            print(self.get_top_local_talkers())
            print("Top Remote Talkers ----------------------------")
            print(self.get_top_remote_talkers())
            print("Active Flows (100) ----------------------------")
            print(self.get_active_flows_paginated(1, 100))
            print("L4 Flow Counters ----------------------------")
            print(self.get_active_l4_proto_flow_counters())
            print("L7 Flow Counters ----------------------------")
            print(self.get_active_l7_proto_flow_counters())
            print("----------------------------")
        except:
            raise ValueError("Invalid interface ID specified")

        




"""
Historical
====================================
The Historical class can be used to retrieve historical traffic data through the
REST API (https://www.ntop.org/guides/ntopng/api/rest/api_v2.html).
"""

import time
import pandas as pd

class Historical:
    """
    Historiacl provides access to historical information including flows and alerts
    
    :param ntopng_obj: The ntopng handle
    """
    def __init__(self, ntopng_obj, ifid=None):
        """
        Construct a new Historical object
        
        :param ntopng_obj: The ntopng handle
        """
        self.ntopng_obj      = ntopng_obj
        self.ifid            = ifid
        self.rest_v2_url     = "/lua/rest/v2"
        self.rest_pro_v2_url = "/lua/pro/rest/v2"

    def get_alert_type_counters(self, epoch_begin, epoch_end):
        """
        Return statistics about the number of alerts per alert type
        
        :param epoch_begin: Start of the time interval (epoch)
        :type epoch_begin: int
        :param epoch_end: End of the time interval (epoch)
        :type epoch_end: int
        :return: Statistics
        :rtype: object
        """
        return(self.ntopng_obj.request(self.rest_v2_url + "/get/alert/type/counters.lua", { "ifid": self.ifid, "status": "historical", "epoch_begin": epoch_begin, "epoch_end": epoch_end }))

    def get_alert_severity_counters(self, epoch_begin, epoch_end):
        """
        Return statistics about the number of alerts per alert severity
        
        :param epoch_begin: Start of the time interval (epoch)
        :type epoch_begin: int
        :param epoch_end: End of the time interval (epoch)
        :type epoch_end: int
        :return: Statistics
        :rtype: object
        """
        return(self.ntopng_obj.request(self.rest_v2_url + "/get/alert/severity/counters.lua", { "ifid": self.ifid, "status": "historical", "epoch_begin": epoch_begin, "epoch_end": epoch_end }))

    def get_alerts(self, alert_family, epoch_begin, epoch_end, select_clause, where_clause, maxhits, group_by, order_by):
        """
        Run queries on the alert database
        
        :param alert_family: The alert family (flow, host, interface, etc)
        :type alert_family: string
        :param epoch_begin: Start of the time interval (epoch)
        :type epoch_begin: int
        :param epoch_end: End of the time interval (epoch)
        :type epoch_end: int
        :param select_clause: Select clause (SQL syntax)
        :type select_clause: string
        :param where_clause: Where clause (SQL syntax)
        :type where_clause: string
        :param maxhits: Max number of results (limit)
        :type maxhits: int
        :param group_by: Group by condition (SQL syntax)
        :type group_by: string
        :param order_by: Order by condition (SQL syntax)
        :type order_by: string
        :return: Query result
        :rtype: object
        """
        return(self.ntopng_obj.request(self.rest_v2_url + "/get/alert/list/alerts.lua", { "ifid": self.ifid, "alert_family": alert_family, "epoch_begin": epoch_begin, "epoch_end": epoch_end,
                                                                                          "select_clause": select_clause, "where_clause": where_clause,
                                                                                          "maxhits_clause": maxhits, "group_by_clause": group_by, "order_by_clause": order_by }))
    
    def get_alerts_stats(self, epoch_begin, epoch_end, host=None):
        """
        Return flow alerts stats
        
        :param epoch_begin: Start of the time interval (epoch)
        :type epoch_begin: int
        :param epoch_end: End of the time interval (epoch)
        :type epoch_end: int
        :param host: Host IP address (optional)
        :type host: string
        :return: Flow alert stats
        :rtype: object
        """
        get_data = { "ifid": self.ifid, "epoch_begin": epoch_begin, "epoch_end": epoch_end }
        
        if host is not None:
            get_data["ip"] = host + ";eq"
        
        return(self.ntopng_obj.request(self.rest_v2_url + "/get/alert/top.lua", get_data))

    def get_flow_alerts_stats(self, epoch_begin, epoch_end):
        """
        Return flow alerts stats
        
        :param epoch_begin: Start of the time interval (epoch)
        :type epoch_begin: int
        :param epoch_end: End of the time interval (epoch)
        :type epoch_end: int
        :return: Flow alert stats
        :rtype: object
        """
        return(self.ntopng_obj.request(self.rest_pro_v2_url + "/get/flow/alert/top.lua", { "ifid": self.ifid, "epoch_begin": epoch_begin, "epoch_end": epoch_end }))

    def get_flow_alerts(self, epoch_begin, epoch_end, select_clause, where_clause, maxhits, group_by, order_by):
        """
        Return flow alerts matching the specified criteria
        
        :param epoch_begin: Start of the time interval (epoch)
        :type epoch_begin: int
        :param epoch_end: End of the time interval (epoch)
        :type epoch_end: int
        :param select_clause: Select clause (SQL syntax)
        :type select_clause: string
        :param where_clause: Where clause (SQL syntax)
        :type where_clause: string
        :param maxhits: Max number of results (limit)
        :type maxhits: int
        :param group_by: Group by condition (SQL syntax)
        :type group_by: string
        :param order_by: Order by condition (SQL syntax)
        :type order_by: string
        :return: Query result
        :rtype: object
        """
        return(self.get_alerts("flow", epoch_begin, epoch_end, select_clause, where_clause, maxhits, group_by, order_by))
    
    def get_active_monitoring_alerts(self, epoch_begin, epoch_end, select_clause, where_clause, maxhits, group_by, order_by):
        """
        Return  alerts matching the specified criteria
        
        :param epoch_begin: Start of the time interval (epoch)
        :type epoch_begin: int
        :param epoch_end: End of the time interval (epoch)
        :type epoch_end: int
        :param select_clause: Select clause (SQL syntax)
        :type select_clause: string
        :param where_clause: Where clause (SQL syntax)
        :type where_clause: string
        :param maxhits: Max number of results (limit)
        :type maxhits: int
        :param group_by: Group by condition (SQL syntax)
        :type group_by: string
        :param order_by: Order by condition (SQL syntax)
        :type order_by: string
        :return: Query result
        :rtype: object
        """
        return(self.get_alerts("active_monitoring", epoch_begin, epoch_end, select_clause, where_clause, maxhits, group_by, order_by))
    
    def get_host_alerts(self, epoch_begin, epoch_end, select_clause, where_clause, maxhits, group_by, order_by):
        """
        Return host alerts matching the specified criteria
        
        :param epoch_begin: Start of the time interval (epoch)
        :type epoch_begin: int
        :param epoch_end: End of the time interval (epoch)
        :type epoch_end: int
        :param select_clause: Select clause (SQL syntax)
        :type select_clause: string
        :param where_clause: Where clause (SQL syntax)
        :type where_clause: string
        :param maxhits: Max number of results (limit)
        :type maxhits: int
        :param group_by: Group by condition (SQL syntax)
        :type group_by: string
        :param order_by: Order by condition (SQL syntax)
        :type order_by: string
        :return: Query result
        :rtype: object
        """
        return(self.get_alerts("host", epoch_begin, epoch_end, select_clause, where_clause, maxhits, group_by, order_by))
    
    def get_interface_alerts(self, epoch_begin, epoch_end, select_clause, where_clause, maxhits, group_by, order_by):
        """
        Return interface alerts matching the specified criteria
        
        :param epoch_begin: Start of the time interval (epoch)
        :type epoch_begin: int
        :param epoch_end: End of the time interval (epoch)
        :type epoch_end: int
        :param select_clause: Select clause (SQL syntax)
        :type select_clause: string
        :param where_clause: Where clause (SQL syntax)
        :type where_clause: string
        :param maxhits: Max number of results (limit)
        :type maxhits: int
        :param group_by: Group by condition (SQL syntax)
        :type group_by: string
        :param order_by: Order by condition (SQL syntax)
        :type order_by: string
        :return: Query result
        :rtype: object
        """
        return(self.get_alerts("interface", epoch_begin, epoch_end, select_clause, where_clause, maxhits, group_by, order_by))
    
    def get_mac_alerts(self, epoch_begin, epoch_end, select_clause, where_clause, maxhits, group_by, order_by):
        """
        Return MAC alerts matching the specified criteria
        
        :param epoch_begin: Start of the time interval (epoch)
        :type epoch_begin: int
        :param epoch_end: End of the time interval (epoch)
        :type epoch_end: int
        :param select_clause: Select clause (SQL syntax)
        :type select_clause: string
        :param where_clause: Where clause (SQL syntax)
        :type where_clause: string
        :param maxhits: Max number of results (limit)
        :type maxhits: int
        :param group_by: Group by condition (SQL syntax)
        :type group_by: string
        :param order_by: Order by condition (SQL syntax)
        :type order_by: string
        :return: Query result
        :rtype: object
        """
        return(self.get_alerts("mac", epoch_begin, epoch_end, select_clause, where_clause, maxhits, group_by, order_by))
    
    def get_network_alerts(self, epoch_begin, epoch_end, select_clause, where_clause, maxhits, group_by, order_by):
        """
        Return Network alerts matching the specified criteria
        
        :param epoch_begin: Start of the time interval (epoch)
        :type epoch_begin: int
        :param epoch_end: End of the time interval (epoch)
        :type epoch_end: int
        :param select_clause: Select clause (SQL syntax)
        :type select_clause: string
        :param where_clause: Where clause (SQL syntax)
        :type where_clause: string
        :param maxhits: Max number of results (limit)
        :type maxhits: int
        :param group_by: Group by condition (SQL syntax)
        :type group_by: string
        :param order_by: Order by condition (SQL syntax)
        :type order_by: string
        :return: Query result
        :rtype: object
        """
        return(self.get_alerts("network", epoch_begin, epoch_end, select_clause, where_clause, maxhits, group_by, order_by))
    
    def get_snmp_alerts(self, epoch_begin, epoch_end, select_clause, where_clause, maxhits, group_by, order_by):
        """
        Return SNMP alerts matching the specified criteria
        
        :param epoch_begin: Start of the time interval (epoch)
        :type epoch_begin: int
        :param epoch_end: End of the time interval (epoch)
        :type epoch_end: int
        :param select_clause: Select clause (SQL syntax)
        :type select_clause: string
        :param where_clause: Where clause (SQL syntax)
        :type where_clause: string
        :param maxhits: Max number of results (limit)
        :type maxhits: int
        :param group_by: Group by condition (SQL syntax)
        :type group_by: string
        :param order_by: Order by condition (SQL syntax)
        :type order_by: string
        :return: Query result
        :rtype: object
        """
        return(self.get_alerts("snmp", epoch_begin, epoch_end, select_clause, where_clause, maxhits, group_by, order_by))
    
    def get_system_alerts(self, epoch_begin, epoch_end, select_clause, where_clause, maxhits, group_by, order_by):
        """
        Return System alerts matching the specified criteria
        
        :param epoch_begin: Start of the time interval (epoch)
        :type epoch_begin: int
        :param epoch_end: End of the time interval (epoch)
        :type epoch_end: int
        :param select_clause: Select clause (SQL syntax)
        :type select_clause: string
        :param where_clause: Where clause (SQL syntax)
        :type where_clause: string
        :param maxhits: Max number of results (limit)
        :type maxhits: int
        :param group_by: Group by condition (SQL syntax)
        :type group_by: string
        :param order_by: Order by condition (SQL syntax)
        :type order_by: string
        :return: Query result
        :rtype: object
        """
        return(self.get_alerts("system", epoch_begin, epoch_end, select_clause, where_clause, maxhits, group_by, order_by))

    def get_user_alerts(self, epoch_begin, epoch_end, select_clause, where_clause, maxhits, group_by, order_by):
        """
        Return User alerts matching the specified criteria
        
        :param epoch_begin: Start of the time interval (epoch)
        :type epoch_begin: int
        :param epoch_end: End of the time interval (epoch)
        :type epoch_end: int
        :param select_clause: Select clause (SQL syntax)
        :type select_clause: string
        :param where_clause: Where clause (SQL syntax)
        :type where_clause: string
        :param maxhits: Max number of results (limit)
        :type maxhits: int
        :param group_by: Group by condition (SQL syntax)
        :type group_by: string
        :param order_by: Order by condition (SQL syntax)
        :type order_by: string
        :return: Query result
        :rtype: object
        """
        return(self.get_alerts("user", epoch_begin, epoch_end, select_clause, where_clause, maxhits, group_by, order_by))

    def timeseries_to_pandas(self, rsp):
        keys = rsp.keys()
        if (("start" in keys) and ("count" in keys) and ("step" in keys)):
            interval = pd.interval_range(rsp['start'], periods=rsp['count'], freq=rsp['step'])
            data = {}
            for serie in rsp['series']:
                data[serie['label']] = serie['data']
            return pd.DataFrame(data, index=interval)
        else:
            return pd.DataFrame([])


    def get_timeseries(self, ts_schema, ts_query, epoch_begin, epoch_end):
        """
        Return timeseries in a pandas DataFrame for a specified schema and query
        See https://pandas.pydata.org/docs/user_guide/10min.html
        
        :param ts_schema: The timeseries schema (e.g. 'host:traffic')
        :type ts_schema: string
        :param ts_query: The timeseries query (e.g. 'ifid:0,host:10.0.0.1')
        :type ts_query: string
        :param epoch_begin: Start of the time interval (epoch)
        :type epoch_begin: int
        :param epoch_end: End of the time interval (epoch)
        :type epoch_end: int
        :return: Timeseries data
        :rtype: object (pandas DataFrame)
        """
        rsp = self.ntopng_obj.post_request(self.rest_v2_url + "/get/timeseries/ts.lua", { "ts_schema": ts_schema, "ts_query": ts_query, "epoch_begin": epoch_begin, "epoch_end": epoch_end })
        return self.timeseries_to_pandas(rsp)

    def get_timeseries_stats(self, ts_schema, ts_query, epoch_begin, epoch_end):
        """
        Return stats from timeseries
        
        :param ts_schema: The timeseries schema (e.g. 'host:traffic')
        :type ts_schema: string
        :param ts_query: The timeseries query (e.g. 'ifid:0,host:10.0.0.1')
        :type ts_query: string
        :param epoch_begin: Start of the time interval (epoch)
        :type epoch_begin: int
        :param epoch_end: End of the time interval (epoch)
        :type epoch_end: int
        :return: Timeseries data
        :rtype: object (pandas DataFrame)
        """
        rsp = self.ntopng_obj.post_request(self.rest_v2_url + "/get/timeseries/ts.lua", { "ts_schema": ts_schema, "ts_query": ts_query, "epoch_begin": epoch_begin, "epoch_end": epoch_end })
        return rsp['statistics']

    def get_timeseries_metadata(self):
        """
        Return timeseries metadata (list all available timeseries)

        :return: Timeseries metadata
        :rtype: object
        """
        return(self.ntopng_obj.request(self.rest_v2_url + "/get/timeseries/type/consts.lua", None))

    def get_host_timeseries(self, host_ip, ts_schema, epoch_begin, epoch_end):
        """
        Return timeseries data in a pandas DataFrame for a specified interface and host
        
        :param host_ip: The host IP
        :type host: string
        :param ts_schema: The timeseries schema
        :type ts_schema: string
        :param epoch_begin: Start of the time interval (epoch)
        :type epoch_begin: int
        :param epoch_end: End of the time interval (epoch)
        :type epoch_end: int
        :return: Timeseries data
        :rtype: object (pandas DataFrame)
        """
        return(self.get_timeseries(ts_schema, "ifid:"+str(self.ifid)+",host:"+host_ip, epoch_begin, epoch_end))

    def get_host_timeseries_stats(self, host_ip, ts_schema, epoch_begin, epoch_end):
        """
        Return timeseries statistics
        
        :param host_ip: The host IP
        :type host: string
        :param ts_schema: The timeseries schema
        :type ts_schema: string
        :param epoch_begin: Start of the time interval (epoch)
        :type epoch_begin: int
        :param epoch_end: End of the time interval (epoch)
        :type epoch_end: int
        :return: Timeseries data
        :rtype: object (pandas DataFrame)
        """
        return(self.get_timeseries_stats(ts_schema, "ifid:"+str(self.ifid)+",host:"+host_ip, epoch_begin, epoch_end))

    def get_interface_timeseries(self, ts_schema, epoch_begin, epoch_end):
        """
        Return timeseries data in a pandas DataFrame for a specified interface
        
        :param ts_schema: The timeseries schema
        :type ts_schema: string
        :param epoch_begin: Start of the time interval (epoch)
        :type epoch_begin: int
        :param epoch_end: End of the time interval (epoch)
        :type epoch_end: int
        :return: Timeseries data
        :rtype: object (pandas DataFrame)
        """
        return(self.get_timeseries(ts_schema, "ifid:"+str(self.ifid), epoch_begin, epoch_end))

    def get_interface_timeseries_stats(self, ts_schema, epoch_begin, epoch_end):
        """
        Return timeseries statistics
        
        :param ts_schema: The timeseries schema
        :type ts_schema: string
        :param epoch_begin: Start of the time interval (epoch)
        :type epoch_begin: int
        :param epoch_end: End of the time interval (epoch)
        :type epoch_end: int
        :return: Timeseries data
        :rtype: object (pandas DataFrame)
        """
        return(self.get_timeseries_stats(ts_schema, "ifid:"+str(self.ifid), epoch_begin, epoch_end))

    def get_flows(self, epoch_begin, epoch_end, select_clause, where_clause, maxhits, group_by, order_by):
        """
        Run queries on the historical flows database (ClickHouse)
        
        :param epoch_begin: Start of the time interval (epoch)
        :type epoch_begin: int
        :param epoch_end: End of the time interval (epoch)
        :type epoch_end: int
        :param select_clause: Select clause (SQL syntax)
        :type select_clause: string
        :param where_clause: Where clause (SQL syntax)
        :type where_clause: string
        :param maxhits: Max number of results (limit)
        :type maxhits: int
        :param group_by: Group by condition (SQL syntax)
        :type group_by: string
        :param order_by: Order by condition (SQL syntax)
        :type order_by: string
        :return: Query result
        :rtype: object
        """
        return(self.ntopng_obj.post_request(self.rest_pro_v2_url + "/get/db/flows.lua", { "ifid": self.ifid, "epoch_begin": epoch_begin, "epoch_end": epoch_end, "select_clause": select_clause, "where_clause": where_clause, "maxhits_clause": maxhits, "group_by_clause": group_by, "order_by_clause": order_by }))

    def get_topk_flows(self, epoch_begin, epoch_end, max_hits, where_clause):
        """
        Retrieve Top-K from the historical flows database
        
        :param epoch_begin: Start of the time interval (epoch)
        :type epoch_begin: int
        :param epoch_end: End of the time interval (epoch)
        :type epoch_end: int
        :param maxhits: Max number of results (limit)
        :type maxhits: int
        :param where_clause: Where clause (SQL syntax)
        :type where_clause: string
        :return: Query result
        :rtype: object
        """
        return(self.ntopng_obj.request(self.rest_pro_v2_url + "/get/db/topk_flows.lua", {"ifid": self.ifid, "begin_time_clause": epoch_begin, "end_time_clause": epoch_end, "maxhits_clause": max_hits, "where_clause": where_clause }))

    def get_top_conversations(self, epoch_begin, epoch_end, host=None):
        """
        Return Top Conversations
        
        :param epoch_begin: Start of the time interval (epoch)
        :type epoch_begin: int
        :param epoch_end: End of the time interval (epoch)
        :type epoch_end: int
        :param host: Host IP address (optional)
        :type host: string
        :return: Top conversations
        :rtype: object
        """
        get_data = { "ifid": self.ifid, "epoch_begin": epoch_begin, "epoch_end": epoch_end, "query_preset": "top_conversations", "order": "DESC", "start": 0, "length": 10, "sort": "bytes", "visible_columns": "cli_ip,srv_ip,bytes" }
        
        if host is not None:
            get_data["ip"] = host + ";eq"
        
        res = self.ntopng_obj.request(self.rest_pro_v2_url + "/get/db/historical_db_search.lua", get_data)
        return(res["records"])

    def get_host_top_protocols(self, host, epoch_begin, epoch_end):
        """
        Return Top protocols for an host

        :param host: Host IP address
        :type host: string
        :param epoch_begin: Start of the time interval (epoch)
        :type epoch_begin: int
        :param epoch_end: End of the time interval (epoch)
        :type epoch_end: int
        :return: Top protocols
        :rtype: object
        """

        get_data = {  "ts_query": "ifid:" + self.ifid + ",host:" + host, "detail_view": "top_protocols", "epoch_begin": epoch_begin, "epoch_end": epoch_end, "new_charts": "true" }

        res = self.ntopng_obj.request(self.rest_pro_v2_url + "/get/host/top/ts_stats.lua", get_data)
        return(res)

    def self_test(self, host):
        try:
            epoch_end   = int(time.time())
            epoch_begin = epoch_end - 3600

            print("Flow alerts ----------------------------")
            print(self.get_flow_alerts(epoch_begin, epoch_end, "*", None, 5, None, "epoch_begin"), None, None)
            print("Active Monitoring alerts ----------------------------")
            print(self.get_active_monitoring_alerts(epoch_begin, epoch_end, "*", None, 5, None, "epoch_begin"), None, None)
            print("Host alertss ----------------------------")
            print(self.get_host_alerts(epoch_begin, epoch_end, "*", None, 5, None, "epoch_begin"), None, None)
            print("Interface alerts ----------------------------")
            print(self.get_interface_alerts(epoch_begin, epoch_end, "*", None, 5, None, "epoch_begin"), None, None)
            print("MAC alerts ----------------------------")
            print(self.get_mac_alerts(epoch_begin, epoch_end, "*", None, 5, None, "epoch_begin"), None, None)
            print("Network alerts ----------------------------")
            print(self.get_network_alerts(epoch_begin, epoch_end, "*", None, 5, None, "epoch_begin"), None, None)
            print("SNMP alerts ----------------------------")
            print(self.get_snmp_alerts(epoch_begin, epoch_end, "*", None, 5, None, "epoch_begin"), None, None)
            print("System alerts ----------------------------")
            print(self.get_system_alerts(epoch_begin, epoch_end, "*", None, 5, None, "epoch_begin"), None, None)
            print("User alerts ----------------------------")
            print(self.get_user_alerts(epoch_begin, epoch_end, "*", None, 5, None, "epoch_begin"), None, None)
            print("Alert counters by type ----------------------------")
            print(self.get_alert_type_counters(epoch_begin, epoch_end))
            print("Alert counters by severity ----------------------------")
            print(self.get_alert_severity_counters(epoch_begin, epoch_end))
            print("Host traffic timeseries ----------------------------")
            print(self.get_timeseries("host:traffic", "ifid:"+str(self.ifid)+",host:"+host, epoch_begin, epoch_end))
            print("Interface traffic timeseries ----------------------------")
            print(self.get_interface_timeseries("iface:traffic_rxtx", epoch_begin, epoch_end))
            print("Host traffic timeseries ----------------------------")
            print(self.get_host_timeseries(host, "host:traffic", epoch_begin, epoch_end))
            print("Interface score timeseries ----------------------------")
            print(self.get_interface_timeseries("iface:score", epoch_begin, epoch_end))
            print("Host flows ----------------------------")
            select_clause = "IPV4_SRC_ADDR,IPV4_DST_ADDR,PROTOCOL,IP_SRC_PORT,IP_DST_PORT,L7_PROTO,L7_PROTO_MASTER"
            where_clause  = "(IP_PROTOCOL_VERSION=4) AND IPV4_SRC_ADDR=(\""+host+"\") AND (PROTOCOL=6) "
            maxhits       = 10 # 10 records max
            print(self.get_flows(epoch_begin, epoch_end, select_clause, where_clause, maxhits, '', ''))
            print("Top K flows ----------------------------")
            print(self.get_topk_flows(epoch_begin, epoch_end, maxhits, None))
            print("Host alerts ----------------------------")
            print(self.get_alerts_stats(epoch_begin, epoch_end, host))
            print("Host Top conversations ----------------------------")
            print(self.get_top_conversations(epoch_begin, epoch_end, host))
            print("Host Top protocols ----------------------------")
            print(self.get_host_top_protocols(host, epoch_begin, epoch_end))
            print("----------------------------")
        except:
            raise ValueError("Invalid interface ID, host or parameters specified")
